local M = {}

local lastY = 0

local doVS = false
local VSF = 0

local function init()
	electrics.values['elevator'] = 0
	electrics.values['st10rpm'] = 0
	electrics.values['st10attitude'] = 0
	electrics.values['st10hdg'] = 0
	electrics.values['st10vs'] = 0
	electrics.values['st10airspeed'] = 0
	electrics.values['st10clocks'] = 0
	electrics.values['st10clockm'] = 0
	electrics.values['st10clockh'] = 0
	electrics.values['st10rudderL'] = 0
	electrics.values['st10rudderR'] = 0
	electrics.values['st10alt100'] = 0
	electrics.values['st10airspeed_raw'] = 0
end

local function reset()
	init()
end

local function set(key,value)
	local keyP = "st10" .. key
	electrics.values[keyP] = value
end

local function updateGFX(dt)
	VSF = VSF + 1
	if VSF == 2 then doVS = true VSF = 0 end
	
	local turnSignals = electrics.values['turnsignal']
	
	local leftRudderPedal = 0
	local rightRudderPedal = 0
	if turnSignals < 0 then leftRudderPedal = math.abs(turnSignals) end
	if turnSignals > 0 then rightRudderPedal = turnSignals end
	
	electrics.values['st10rudderL'] = leftRudderPedal
	electrics.values['st10rudderR'] = rightRudderPedal
	
	local curPos = obj:getPosition()
	
		
	local elev = electrics.values["brake"] - electrics.values["throttle"]
	electrics.values['elevator'] = elev * -1
	
   	local dirVector = obj:getDirectionVector()
 	local dirVectorUp = obj:getDirectionVectorUp()
   	local roll = dirVectorUp.x * -dirVector.y + dirVectorUp.y * dirVector.x
	local rollProcessed = math.deg(roll) 
	electrics.values['st10attitude'] = rollProcessed * -1
	electrics.values['st10rpm'] = drivetrain.wheelInfo[3].angularVelocity
	
	electrics.values['st10hdg'] = (math.deg(obj:getDirection()) % 360)
	

	
	if doVS then
		local cAltFeet = curPos.z * 3.2808
		local altDiff = cAltFeet - lastY
		
		local feetPerMinute = altDiff * (60000 / dt)
		electrics.values['st10vs'] = feetPerMinute / 30000
		
		lastY = curPos.z * 3.28083

		doVS = false
	end
	
	electrics.values['st10alt100'] = (lastY / 1000) % 1
	
	local airspeed = obj:getVelocity():length() * 1.9438444924406
	
	--calibration
	local airspeedP = airspeed - 20
	if airspeedP < 0 then airspeedP = 0 end
	
	electrics.values['st10airspeed']  = (airspeedP / 140)
	
	if airspeed > 160 then airspeed = 160 end
	electrics.values['st10airspeed_raw'] = airspeed
	
	local dateTime = os.date("*t", os.time())
	
	local psec = dateTime.sec/ 60
	local pmin = (dateTime.min + psec) / 60
	local ph = ((dateTime.hour % 12) + (pmin)) / 12
	
	
	--prop functions : st10clocks st10clockm st10clockh
	--prop factor should be 360
	electrics.values['st10clocks'] = psec
	electrics.values['st10clockm'] = pmin 
	electrics.values['st10clockh'] =  ph 
	
end

-- public interface
M.init      = init
M.set = set
M.reset     = reset
M.updateGFX = updateGFX

return M